# Table of contents

* [Introduction](README.md)
* [Core Concepts](core_concepts/README.md)
  * [Best Practices](core_concepts/best_practices.md)
  * [In-depth](core_concepts/tour.md)
* [Tutorial](tut/README.md)
  * [Getting Started](tut/getting-started.md)
  * [Reading from a Database](tut/executing-queries.md)
  * [Storing Data in a Database](tut/storing-data.md)
  * [Configuration and Writing Tests](tut/writing-tests.md)
  * [Authentication and Authorization with OAuth 2.0](tut/oauth2.md)
  * [Deploying a Conduit Application](tut/deploying-and-other-fun-things.md)
* [Deploying Conduit](deploy/README.md)
  * [Deploy on a Local Machine](deploy/deploy_local.md)
  * [Deploying on Docker](deploy/deploy_docker.md)
  * [Creating an Executable](deploy/build.md)
  * [Script](deploy/script.md)
* [HTTP/REST](http/README.md)
  * [Serving Files and Caching](http/serving_files.md)
  * [Uploading Files](http/file_upload.md)
  * [Handling Requests: Fundamentals](http/controller.md)
  * [Serializing Request and Response Bodies](http/request_and_response.md)
  * [ResourceController](http/resource_controller.md)
  * [Using Websockets in Conduit](http/websockets.md)
  * [Routing](http/routing.md)
* [Application](application/README.md)
  * [Configuring an Application and its Environment](application/configure.md)
  * [Starting and Stopping an Application](application/channel.md)
  * [Multi-threading in Conduit](application/threading.md)
  * [Application and Project Structure](application/structure.md)
* [Database access](db/README.md)
  * [Database Migration and Tooling](db/db_tools.md)
  * [Connecting to a Database from Conduit](db/connecting.md)
  * [Database Transactions](db/transactions.md)
  * [Advanced Queries: Filtering, Joins, Paging and Reduce](db/advanced_queries.md)
  * [Validating Data](db/validations.md)
  * [Inserting, Updating, Deleting and Fetching Objects](db/executing_queries.md)
  * [JSON Document Storage](db/json_columns.md)
  * [Modeling Data](db/modeling_data.md)
  * [ManagedObject Serialization and Deserialization](db/serialization.md)
  * [MySql](db/mysql.md)
* [Authentication](auth/README.md)
  * [Creating AuthServers to Authenticate and Authorize](auth/server.md)
  * [Manage OAuth 2.0 Clients](auth/cli.md)
  * [Issue Access Tokens with AuthController](auth/controllers.md)
  * [auth\_scopes](auth/auth_scopes.md)
  * [Securing Routes with Authorizer](auth/authorizer.md)
* [Testing](testing/README.md)
  * [Using Conduit when Writing Client Applications](testing/clients.md)
  * [Testing in Conduit](testing/tests.md)
  * [Testing Applications That Use ORM and OAuth 2.0](testing/mixins.md)
  * [Mocking External Services](testing/mock.md)
  * [Debugging Conduit Applications](testing/debugger.md)
* [Snippets](snippets/README.md)
  * [Conduit Snippets](snippets/index.md)
  * [Conduit HTTP Snippets](snippets/http.md)
  * [Conduit ORM Snippets](snippets/orm.md)
  * [Conduit Authorization and Authentication Snippets](snippets/auth.md)
* [OpenAPI](openapi/README.md)
  * [Document Components](openapi/components.md)
  * [Documenting Endpoint Controllers](openapi/endpoint.md)
  * [Creating OpenAPI Documents](openapi/cli.md)
  * [Documenting Middleware Controllers](openapi/middleware.md)

